#include <QCoreApplication>
#include <QTimer>
#include <QDebug>

#include <signal.h>
#include <inttypes.h>

#include "decode_simple.h"

extern "C" {
    #include "libavutil/video_enc_params.h"
    #include "libavcodec/avcodec.h"
}

#pragma comment(lib, "Secur32.lib")
#pragma comment(lib, "ws2_32.lib")
#pragma comment(lib, "bcrypt.lib")
#pragma comment(lib, "Mfplat.lib")
#pragma comment(lib, "Mfuuid.lib")
#pragma comment(lib, "Strmiids.lib")
#pragma comment(lib, "ole32.lib")

#pragma comment(lib, "User32.lib")

#pragma comment(lib, "libavcodec.a")
#pragma comment(lib, "libavdevice.a")
#pragma comment(lib, "libavfilter.a")
#pragma comment(lib, "libavformat.a")
#pragma comment(lib, "libavutil.a")
#pragma comment(lib, "libpostproc.a")
#pragma comment(lib, "libswresample.a")
#pragma comment(lib, "libswscale.a")

void save_vframe(AVFrame *avFrame, int frameNumber)
{
    char filename[32];
    sprintf(filename, "frame%d.yuv", frameNumber);
    FILE *fDump = fopen(filename, "ab");

    uint32_t pitchY = avFrame->linesize[0];
    uint32_t pitchU = avFrame->linesize[1];
    uint32_t pitchV = avFrame->linesize[2];

    uint8_t *avY = avFrame->data[0];
    uint8_t *avU = avFrame->data[1];
    uint8_t *avV = avFrame->data[2];

    for (int32_t i = 0; i < avFrame->height; i++) {
        fwrite(avY, avFrame->width, 1, fDump);
        avY += pitchY;
    }

    for (int32_t i = 0; i < avFrame->height/2; i++) {
        fwrite(avU, avFrame->width/2, 1, fDump);
        avU += pitchU;
    }

    for (int32_t i = 0; i < avFrame->height/2; i++) {
        fwrite(avV, avFrame->width/2, 1, fDump);
        avV += pitchV;
    }

    fclose(fDump);
}

static int process_frame(DecodeContext *dc, AVFrame *frame)
{
    static bool _show_info = true;

    if ((NULL == dc) || (NULL == frame)) {
        return -1;
    }

    if (true == _show_info) {
        fprintf(stderr, "%d x %d\n", dc->frame->width, dc->frame->height);
        _show_info = false;
    }
    save_vframe(dc->frame, dc->decoder->frame_number);

    return 0;
}

int run_main(void)
{
    DecodeContext dc;

    unsigned int stream_idx, max_frames;
    const char *filename;
    int ret = 0;

    filename = "E:/_001/test.avi";
    stream_idx  = 0;
    max_frames  = 50;

    ret = ds_open(&dc, filename, stream_idx);
    if (ret < 0) {
        goto finish;
    }

    dc.process_frame = process_frame;
    dc.max_frames    = max_frames;

    ret = ds_run(&dc);

finish:
    ds_free(&dc);
    return ret;

    return 0;
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    QTimer::singleShot(0,[](){
         run_main();
         QCoreApplication::exit(0);
    });

    return a.exec();
}
